import { effects } from "redux-saga";
import {
  isSendingAuthRequest,
  setAuthRequestError,
  setAuthRequestSuccess,
  clearAuthRequestError,
  clearAuthRequestSuccess,
  setSelfUserToken,
  setFormEmail,
  setFormPassword,
  setFormPasswordConfirmation,
  setFormOldPassword,
  setClearAuthState
} from "../actions/auth/reducer.actions";
import {
  setSelfUser,
  setClearUserState
} from "../actions/user/reducer.actions";
import { setClearWorktypeState } from "../actions/worktype/reducer.actions";
import {
  registerUser,
  verifyEmail,
  loginUser,
  logoutUser,
  changePassword,
  forgotPassword,
  resetPassword
} from "../api/auth.api";

function* registerUserCall(postBody) {
  yield effects.put(isSendingAuthRequest(true));
  const { email, password1, password2 } = postBody;
  try {
    return yield effects.call(registerUser, email, password1, password2);
  } catch (exception) {
    yield effects.put(setAuthRequestError(exception));
    return false;
  } finally {
    yield effects.put(isSendingAuthRequest(false));
  }
}

function* verifyEmailCall(postBody) {
  yield effects.put(isSendingAuthRequest(true));
  const { emailKey } = postBody;
  try {
    return yield effects.call(verifyEmail, emailKey);
  } catch (exception) {
    yield effects.put(setAuthRequestError(exception));
    return false;
  } finally {
    yield effects.put(isSendingAuthRequest(false));
  }
}

function* loginUserCall(postBody) {
  yield effects.put(isSendingAuthRequest(true));
  const { email, password } = postBody;
  try {
    return yield effects.call(loginUser, email, password);
  } catch (exception) {
    yield effects.put(setAuthRequestError(exception));
    return false;
  } finally {
    yield effects.put(isSendingAuthRequest(false));
  }
}

function* logoutUserCall() {
  yield effects.put(setClearAuthState());
  yield effects.put(setClearUserState());
  yield effects.put(setClearWorktypeState());
  yield effects.call(logoutUser);
}

function* changePasswordCall(postBody) {
  yield effects.put(isSendingAuthRequest(true));
  const { new_password1, new_password2, old_password } = postBody;
  try {
    return yield effects.call(
      changePassword,
      new_password1,
      new_password2,
      old_password
    );
  } catch (exception) {
    yield effects.put(setAuthRequestError(exception));
    return false;
  } finally {
    yield effects.put(isSendingAuthRequest(false));
  }
}

function* forgotPasswordCall(postBody) {
  yield effects.put(isSendingAuthRequest(true));
  const { email } = postBody;
  try {
    return yield effects.call(forgotPassword, email);
  } catch (exception) {
    yield effects.put(setAuthRequestError(exception));
    return false;
  } finally {
    yield effects.put(isSendingAuthRequest(false));
  }
}

function* resetPasswordCall(postBody) {
  yield effects.put(isSendingAuthRequest(true));
  const { uid, token, new_password1, new_password2 } = postBody;
  try {
    return yield effects.call(
      resetPassword,
      uid,
      token,
      new_password1,
      new_password2
    );
  } catch (exception) {
    yield effects.put(setAuthRequestError(exception));
    return false;
  } finally {
    yield effects.put(isSendingAuthRequest(false));
  }
}

export function* registerUserFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  const wasSuccessful = yield effects.call(registerUserCall, request.data);
  if (wasSuccessful) {
    yield effects.put(setAuthRequestSuccess(wasSuccessful));
    yield effects.put(clearAuthRequestError());
    yield effects.put(setFormEmail(""));
    yield effects.put(setFormPassword(""));
    yield effects.put(setFormPasswordConfirmation(""));
  }
}

export function* verifyEmailFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  const wasSuccessful = yield effects.call(verifyEmailCall, request.data);
  if (wasSuccessful) {
    yield effects.put(setAuthRequestSuccess(wasSuccessful));
    yield effects.put(clearAuthRequestError());
  }
}

export function* loginUserFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  const wasSuccessful = yield effects.call(loginUserCall, request.data);
  if (wasSuccessful) {
    yield effects.put(setSelfUserToken(wasSuccessful.key));
    yield effects.put(setAuthRequestSuccess(wasSuccessful));
    yield effects.put(clearAuthRequestError());
    yield effects.put(setFormEmail(""));
    yield effects.put(setFormPassword(""));
    yield effects.put(setFormPasswordConfirmation(""));
  }
}

export function* logoutUserFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  yield effects.call(logoutUserCall);
  yield effects.put(setSelfUserToken(""));
  yield effects.put(setSelfUser({}));
}

export function* changePasswordFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  const wasSuccessful = yield effects.call(changePasswordCall, request.data);
  if (wasSuccessful) {
    yield effects.put(setAuthRequestSuccess(wasSuccessful));
    yield effects.put(clearAuthRequestError());
    yield effects.put(setFormOldPassword(""));
    yield effects.put(setFormPassword(""));
    yield effects.put(setFormPasswordConfirmation(""));
  }
}

export function* forgotPasswordFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  const wasSuccessful = yield effects.call(forgotPasswordCall, request.data);
  if (wasSuccessful) {
    yield effects.put(setAuthRequestSuccess(wasSuccessful));
    yield effects.put(clearAuthRequestError());
  }
}

export function* resetPasswordFlow(request) {
  yield effects.put(clearAuthRequestSuccess());
  yield effects.put(clearAuthRequestError());
  const wasSuccessful = yield effects.call(resetPasswordCall, request.data);
  if (wasSuccessful) {
    yield effects.put(setAuthRequestSuccess(wasSuccessful));
    yield effects.put(clearAuthRequestError());
    yield effects.put(setFormPassword(""));
    yield effects.put(setFormPasswordConfirmation(""));
  }
}
